package downloader.fc;

import javax.swing.SwingWorker;

import java.util.concurrent.locks.Lock;
import java.util.concurrent.locks.ReentrantLock;

import java.net.URL;
import java.net.URLConnection;
import java.net.MalformedURLException;

import java.io.File;
import java.io.BufferedInputStream;
import java.io.FileOutputStream;
import java.io.IOException;

import java.nio.file.Files;
import java.nio.file.StandardCopyOption;

import java.awt.Toolkit;


public class Downloader extends SwingWorker<String, Void> {
	public static final int CHUNK_SIZE = 1024;
	Lock pause = new ReentrantLock();
	
	URL url;
	int content_length;
	BufferedInputStream in;
	
	String filename;
	File temp;
	FileOutputStream out;
	
	public Downloader(String uri) {
		try {
			url = new URL(uri);
			
			URLConnection connection = url.openConnection();
			content_length = connection.getContentLength();
			
			in = new BufferedInputStream(connection.getInputStream());
			
			String[] path = url.getFile().split("/");
			filename = path[path.length-1];
			temp = File.createTempFile(filename, ".part");
			out = new FileOutputStream(temp);
		}
		catch(MalformedURLException e) { throw new RuntimeException(e); }
		catch(IOException e) { throw new RuntimeException(e); }
	}
	
	public String toString() {
		return url.toString();
	}
	
	public String doInBackground() throws InterruptedException {
		byte buffer[] = new byte[CHUNK_SIZE];
		int size = 0;
		int count = 0;
		
		while(true) {
			if(isCancelled()) {
				System.out.println("cancel");
				break;
			}
			
			pause.lock();
			try {
				count = in.read(buffer, 0, CHUNK_SIZE);
			}
			catch(IOException e) {
				pause.unlock();
				continue;
			}
			
			if(count < 0) {
				pause.unlock();
				break;
			}

			try {
				out.write(buffer, 0, count);
			}
			catch(IOException e) {
				pause.unlock();
				continue;
			}
			pause.unlock();
			
			size += count;
			setProgress(100*size/content_length);
			Thread.sleep(1000);
		}
				
		if(size < content_length) {
			temp.delete();
			throw new InterruptedException();
		}
		
		File result = new File(filename);
		try {
			out.close();
			Files.move(temp.toPath(), result.toPath(), StandardCopyOption.REPLACE_EXISTING);
		} catch(IOException e) {
			throw new RuntimeException(e);
		}
		return result.getAbsolutePath();
	}
	
	public void pause() { pause.lock(); }
	public void play()  { pause.unlock(); }
	
	public void done() {
		Toolkit.getDefaultToolkit().beep();
	}
}
